import Router from "koa-router";
import { v4 as uuidv4 } from "uuid";
import sharp from "sharp";

import { createAvatar, Style } from "@dicebear/core";
import * as openPeeps from "@dicebear/open-peeps";
import * as pixelArt from "@dicebear/pixel-art";
import * as thumbs from "@dicebear/thumbs";
import * as botttsNeutral from "@dicebear/bottts-neutral";
import { z } from "zod";

const DENSITY = 72;
const avatars: Record<string, Style<any> & { defaultSize: number }> = {
  "open-peeps": { ...openPeeps, defaultSize: 704 },
  "pixel-art": { ...pixelArt, defaultSize: 16 },
  "bottts-neutral": { ...botttsNeutral, defaultSize: 120 },
  thumbs: { ...thumbs, defaultSize: 100 },
};

const schema = z.object({
  style: z.enum(Object.keys(avatars) as [string, ...string[]]).default("thumbs"),
  size: z.number().min(16).max(512).default(128),
  seed: z.string().min(1).max(256).default(uuidv4()),
});

const router = new Router();

router.get("/list", async (ctx) => {
  const styles = Object.keys(avatars).map((key) => ({
    ...avatars[key].meta,
    key,
  }));
  ctx.body = { styles };
});

router.use(async (ctx, next) => {
  ctx.set({
    Expires: new Date(Date.now() + 604800000).toUTCString(),
  });
  ctx.type = "image/png";
  await next();
});

router.get("/random", async (ctx) => {
  const { style, size, seed } = schema.parse(ctx.query);
  const density = (DENSITY * size) / avatars[style].defaultSize;
  ctx.body = sharp(
    Buffer.from(createAvatar(avatars[style], { seed }).toString()),
    { density }
  ).png();
});

router.get("/:seed", async (ctx) => {
  const { style, size, seed } = schema.parse({
    ...ctx.query,
    ...ctx.params,
  });
  const density = (DENSITY * size) / avatars[style].defaultSize;
  ctx.body = sharp(
    Buffer.from(createAvatar(avatars[style], { seed }).toString()),
    {
      density,
    }
  ).png();
});

export default router;
